define(function (require) {
    const Backbone = require('backbone'),
        BackboneRadio = require('backbone.radio'),
        uuid = require('uuid'),
        CONSTANTS = {
            CHANNEL_NAME: "course_search",
            EVENT_SEARCH_CHANGE: "course_search_change",
            EVENT_SEARCH_WAS_HIDDEN: "course_search_was_hidden",
        },
        courseSearchChannel = BackboneRadio.channel(CONSTANTS.CHANNEL_NAME);

    const SearchModel = Backbone.Model.extend({

        model: Backbone.Model,

        initialize: function (model, options) {
            Backbone.Model.prototype.initialize.apply(this, arguments);
            this.options = options;
            this.on('change:confirmedSearchPhrase', this.onChangeSearchPhrase);
            this.on('change:searchIsOpen', this.onChangeSearchIsOpen);
        },

        parse: function (results) {
            return {'suggestions': [...results]};
        },

        /**
         * Add a function that will be called when the value in the sidebar search
         * input changes. This will also be called if the sidebar search panel is opened.
         * @param searchChangeHandler function with a single parameter `searchPhrase`
         */
        addSearchChangeListener: (searchChangeHandler) => {
            if (typeof searchChangeHandler === "function") {
                courseSearchChannel.on(
                    CONSTANTS.EVENT_SEARCH_CHANGE, searchChangeHandler
                )
            } else {
                throw "addSearchChangeListener requires a function parameter";
            }
        },

        /**
         * Add a function that will be called when the sidebar search panel is closed
         * or switched to a new panel.
         * @param searchWasHiddenHandler
         */
        addSearchWasHiddenListener: (searchWasHiddenHandler) => {
            if (typeof searchWasHiddenHandler === "function") {
                courseSearchChannel.on(
                    CONSTANTS.EVENT_SEARCH_WAS_HIDDEN, searchWasHiddenHandler
                )
            } else {
                throw "addSearchWasHiddenListener requires a function parameter";
            }
        },

        /**
         * After the sidebar search input changes the searchPhrase, notify all listeners
         * that have been added with addSearchChangeListener()
         * @param model - this model
         * @param searchPhrase - the new searchPhrase
         */
        onChangeSearchPhrase: (model, searchPhrase) => {
            if (searchPhrase) {
                window.globals.searchPhrase = searchPhrase;
                courseSearchChannel.trigger(
                    CONSTANTS.EVENT_SEARCH_CHANGE, searchPhrase
                );
            }
        },

        /**
         * After the sidebar search closes, notify all the listeners that have been added with
         * addSearchWasHiddenListener. When the sidebar opens, notify all the listeners
         * that have been added with addSearchChangeListener.
         * @param model - this model
         * @param searchIsOpening - boolean, True iff search is opening up in side panel.
         */
        onChangeSearchIsOpen: (model, searchIsOpening) => {
            if (searchIsOpening) {
                courseSearchChannel.trigger(CONSTANTS.EVENT_SEARCH_CHANGE, model.get('searchPhrase'));
            } else {
                courseSearchChannel.trigger(CONSTANTS.EVENT_SEARCH_WAS_HIDDEN, {});
            }
        },

        /**
         * Override Backbone.Model.url
         * http://backbonejs.org/#Model-url
         * TODO: support admin preview passing a wave id of zero
         * @return {string} url to get with this.fetch
         */
        url: function () {
            const {waveId, cohortId, courseInstanceId, conceptId} = this.options,
                searchPhrase = encodeURIComponent(this.get('searchPhrase')),
                queryParam = `searchPhrase=${searchPhrase}`,
                cohortIdParam = `cohort_id=${cohortId}`,
                waveIdParam = `wave_id=${waveId}`,
                conceptIdParam = `concept_id=${conceptId}`,
                courseInstanceIdParam = `ci_ids=${courseInstanceId}`,
                suggestParam = `suggest=1`,
                queryParams = [queryParam, waveIdParam, cohortIdParam, conceptIdParam, courseInstanceIdParam, suggestParam].join("&");
            return `/coursesearch?${queryParams}`;
        }
    });

    return SearchModel;
});
