define([
    'jquery',
    'domReady',
    'backbone.radio'
], function (
    $,
    domReady,
    BackboneRadio
) {

    var EventMetric = new function() {
        /*
        * Navigation metric singleton object.
        * Keep global data definition and methods required for metrics here.
        */
        var that = this;
        this.metricsChannel = BackboneRadio.channel("metrics");
        this.page_name = window.globals.metric_page_name;
        this.metricsActivated = window.globals.eventMetricsActive;
        this.page_title = $("title").text();
        this.channelEventName = "event_metric_package";

        // Click events for elements being tracked.
        $('li[role="menuitem"]').on('click', function (evt) {
            that.createHeaderClickMetric(evt);
        });
        this.metricsChannel.on('profile_view_open', function (item) {
            that.createProfileViewMetric(item);
        });
        this.metricsChannel.on('sidebar_item_click', function (item) {
            that.createSidebarClickMetric(item);
        });
        this.metricsChannel.on('search', function (item, search_area, feature_name) {
            that.createSearchMetric(item, search_area, feature_name);
        });
        this.metricsChannel.on('course_search', function (courseSearchMetricObj) {
            that.createCourseSearchMetric(courseSearchMetricObj);
        });

        this.metricsChannel.on('activity_tab_click', function (item) {
            that.createActivityTabClickMetric(item);
        });
        this.metricsChannel.on('syllabus_click', function (wave_id, module_id, lesson_id, concept_id) {
            that.createSyllabusClickMetric(wave_id, module_id, lesson_id, concept_id);
        });
        this.metricsChannel.on('keep_learning_selected', function(item) {
            that.createKeepLearningMetric(item);
        });

        // Make sure this is inline with the categories defined on backend.
        this.metric_categories = {
            TIME_ON: "time_on",
            HEADER_CLICK: "header_click",
            PROFILE_VIEW: "profile_view",
            SIDEBAR_CLICK: "sidebar_click",
            SEARCH: "search",
            ACTIVITY_TAB_CLICK: "activity_tab_click",
            SYLLABUS_CLICK: "syllabus_click",
            CATEGORY_KEEP_LEARNING: "keep_learning_selected"
        };

        this.sendMetricPackage = function(metric_data) {
            /*
            * Check is metrics allowed before sending package.
            * Add common data to the package being sent.
            */
            if (this.metricsActivated) {
                metric_data = this.addCommonMetricData(metric_data);
                this.metricsChannel.trigger(this.channelEventName, metric_data);
            }
        };

        this.addCommonMetricData = function(metric_data) {
            /*
            * Add common data shared across all metrics to the metric package.
            * Keep event_metric_package to true so channel knows its metrics coming up.
            */
            metric_data.channel = 'event_metric_package';
            metric_data.page_name = this.page_name;
            metric_data.page_title = this.page_title;

            if (window.globals) {
                if (window.globals.courseInstanceId)
                    metric_data.course_instance_id = window.globals.courseInstanceId;
                if (window.globals.concept)
                    metric_data.concept_id = window.globals.concept;
            }

            return metric_data;
        };

        this.createTimeOnConnectMetric = function () {
            // Time-on metric for calculating how much time user spent on the site.
            this.sendMetricPackage({
                category: this.metric_categories.TIME_ON
            });
        };

        this.createHeaderClickMetric = function (evt) {
            // Metric for showing number of clicks on items in the header.
            var item = $(evt.target).closest('li').data('click-metric');
            this.sendMetricPackage({
                category: this.metric_categories.HEADER_CLICK,
                item: item
            });
        };

        this.createProfileViewMetric = function (item) {
            // Metric for showing number of viewed profiles.
            this.sendMetricPackage({
                category: this.metric_categories.PROFILE_VIEW,
                item: item
            });
        };

        this.createSidebarClickMetric = function (item) {
            // Metric for showing number of clicks on sidebar items.
            this.sendMetricPackage({
                category: this.metric_categories.SIDEBAR_CLICK,
                item: item
            });
        };

        this.createSearchMetric = function (item, search_area, feature_name) {
            // Metric for catch searched text.
            this.sendMetricPackage({
                category: this.metric_categories.SEARCH,
                item: item,
                search_area: search_area,
                feature_name: feature_name
            });
        };

        /**
         * A more specific version of createSearchMetric for
         * course search. Sent only from `SearchView.js`.
         * @param courseSearchMetricObj - object with attributes to send
         */
        this.createCourseSearchMetric = function(courseSearchMetricObj) {
            this.sendMetricPackage({
                category: this.metric_categories.SEARCH,
                ...courseSearchMetricObj
            });
        };

        this.createActivityTabClickMetric = function (item) {
            // Metric for showing number of clicks on activity feed.
            this.sendMetricPackage({
                category: this.metric_categories.ACTIVITY_TAB_CLICK,
                item: item
            });
        };

        this.createSyllabusClickMetric = function (wave_id, module_id, lesson_id, clicked_concept_id) {
            // Metric for showing number of clicks on activity feed.
            this.sendMetricPackage({
                category: this.metric_categories.SYLLABUS_CLICK,
                wave_id: wave_id,
                module_id: module_id,
                lesson_id: lesson_id,
                clicked_concept_id: clicked_concept_id
            });
        };

        /**
         * Invoked from user-session-timeout.js
         * Marks user selecting to continue learning to avoid session timeout.
         * @param item
         */
        this.createKeepLearningMetric = function (item) {
            this.sendMetricPackage({
                category: this.metric_categories.CATEGORY_KEEP_LEARNING,
                item: item || ''
            });
        };
    };

    // Require.js equivalent of jquery dom ready event.
    domReady(function () {
        // Check is socket available and open or wait for its connect event.
        // If its not available something went wrong and we are not doing anything.
        if (typeof window.hbx_socket != "undefined") {
            if (window.hbx_socket.readyState === window.hbx_socket.OPEN) {
                EventMetric.createTimeOnConnectMetric();
            } else {
                window.hbx_socket.onopen = function () {
                    EventMetric.createTimeOnConnectMetric();
                };
            }
        }
    });

});