define([
    'jquery',
    'fabric',
    'stickynotes',
    'modernizr_pointerevents',
    'jquery_resize'
], function($) {

    function StudentTools() {

        function Stack() {
            this.MAX = 20;
            this.array = [];

            this.pop = function() {
                return this.array.pop();
            };

            this.push = function(item) {
                // Check array size, but never lose track of the initial state.
                if (this.array.length >= this.MAX) {
                    var initial = this.array.shift();
                    this.array[0] = initial;
                }
                this.array.push(item);
            };

            this.clear = function() {
                /* Clear current state of action stack */
                this.array = [];
            };

            this.updateInitial = function(state) {
                var initialState = $.extend(this.array[0], state);
                this.clear();
                initialState.initial = true;
                this.array.push(initialState);
            };

            /* Check if there are changes done by the user */
            this.clean = function(tools) {

                // collect last initial state of student tools
                var initial = null;
                for (var i = this.array.length; i > 0; --i) {
                    if (this.array[i - 1].initial == true) {
                        initial = this.array[i - 1];
                        break;
                    }
                }

                // collect current state of student tools
                var freehand = tools.exportFreehand();
                var note = tools.exportNote();

                // check is the current state different from last initial
                if (initial != null)
                    if (freehand == initial.freehand && note == initial.note)
                        return true;
                return false;
            };
        }

        // Initial set of data
        this.canvas_element = 'hbx_canvas';
        this.active_color = $('#color_choice [data-sticky="green"]').data("color");
        this.active_sticky_class = $('#color_choice [data-sticky="green"]').data("sticky");
        this.sticky_container = $("#sticky-container");
        this.freehand_clear_element = $('#freehandClearContainer');
        this.default_line_width = 1;
        this.state_stack = new Stack();
        this.pointer_events = Modernizr.pointerevents;
        this.autoSaveInterval = 5000;   // in miliseconds
        this.tool_freehand_status = false;
        this.tool_freehand_line_width = undefined;
        this.html_container = $("#hbx_teaching_elements");
        this.sticky_options = { notes: [],
            logCallback: this.notesCallback,
            toolsObject: this
        };

        this.exists = function(data) {
            if (typeof data === 'undefined')
                return false;
            return true;
        };

        // LET'S GET READY TO RUMBLE!!!
        this.initializeTools();
    }

    StudentTools.prototype.initializeTools = function() {
        var _this = this;

        this.canvas = new fabric.Canvas(this.canvas_element);
        this.canvas.setWidth(800);
        this.canvas_container = $("#" + this.canvas_element).parent();
        this.canvas.setHeight(this.html_container.height() + 30);           // + 30 cause of container padding
        this.canvas_container.height(this.html_container.height() + 30);    // + 30 cause of container padding
        this.sticky_container.height(this.html_container.height());

        // load canvas data if any
        if (this.exists(window.tools_drawings))
            this.importFreehand(window.tools_drawings);

        this.sticky_container.stickyNotes(this.sticky_options);
        // load sticky data if any
        if (this.exists(window.tools_notes))
            this.importNote(window.tools_notes);

        // initialize action stack
        this.saveState("all", "initial");

        // load required callbacks
        this.freehandCallback();

        // check if height on container changes and react on it
        $("#hbx_teaching_elements").resize(function(e) {
            var new_height = _this.html_container.height();
            _this.sticky_container.height(new_height);
            _this.canvas.setHeight(new_height + 30);           // + 30 cause of container padding
            _this.canvas_container.height(new_height + 30);    // + 30 cause of container padding
        });

        // set initial focus on html content
        this.focus();

        // set up auto save on regular intervals
        setInterval(function() {

            // see if freehand is active before this so it can be turned on again after
            var tool_freehand_status = _this.tool_freehand_status;
            var tool_freehand_line_width = _this.tool_freehand_line_width;

            var currentState = _this.saveState("all", "initial", "no_save");
            var oldLen = _this.state_stack.array.length;
            var oldState = _this.state_stack.array[oldLen - 1];
            oldState.initial = undefined;

            if (JSON.stringify(currentState) !== JSON.stringify(oldState)) {
                _this.save();

                // get freehand to initial state
                _this.freehandMode(tool_freehand_status, tool_freehand_line_width);
                if (tool_freehand_status == true)
                    _this.focus("canvas");
                else
                    _this.focus();
            }

        }, this.autoSaveInterval);

        // Trashing freehand selections (small trash icon that appears next to drawings when selected)
        $("#freehandClearContainer").on("click", function() {
            _this.clearSelectedFreehand();
            $(this).hide();
        });
    };

    StudentTools.prototype.focus = function(element) {
        /* check does browser support pointer events, if not use z-index */

        if (this.pointer_events) {
            if (element == "canvas")
                this.canvas_container.css("pointer-events", "auto");
            else
                this.canvas_container.css("pointer-events", "none");
        }
        else {
            if (element == "canvas") {
                this.html_container.css("z-index", 0);
                this.canvas_container.css("z-index", 101);
            }
            else {
                this.html_container.css("z-index", 1);
                this.canvas_container.css("z-index", 0);
            }
        }
    };

    StudentTools.prototype.calcOffset = function() {
        this.canvas.calcOffset();
    };

    StudentTools.prototype.freehandMode = function(status, line_width) {
        /* drawing capability, default is true */
        line_width = typeof line_width !== 'undefined' ? line_width : this.default_line_width;

        if (status == false) {
            this.canvas.isDrawingMode = status;
            this.tool_freehand_status = false;
            this.focus();
        }
        else {
            this.focus("canvas");
            if (status == "edit") {
                this.canvas.isDrawingMode = false;
                this.tool_freehand_status = "edit";
            }
            else {
                this.canvas.isDrawingMode = true;
                this.tool_freehand_status = true;
                this.canvas.freeDrawingBrush.color = this.active_color;
                this.canvas.freeDrawingBrush.width = line_width;
                this.tool_freehand_line_width = line_width;
            }
        }
    };

    StudentTools.prototype.clearFreehand = function() {
        this.canvas.clear();
    };

    StudentTools.prototype.clearSelectedFreehand = function() {
        /* Delete only current selected freehand objects */
        if (this.canvas.getActiveGroup()) {
            this.saveState("freehand");
            var _this = this;
            this.canvas.getActiveGroup().forEachObject(function(o) {
                _this.canvas.remove(o)
            });
            this.canvas.discardActiveGroup().renderAll();
        }
        else {
            this.saveState("freehand");
            this.canvas.remove(this.canvas.getActiveObject());
        }
    };

    StudentTools.prototype.showFreehandOption = function(type) {
        var selected = "";
        if (type == "object")
            selected = this.canvas.getActiveObject();
        else
            selected = this.canvas.getActiveGroup();

        this.repositionFreehandOption(selected.left, selected.top, selected.width, selected.height);
        this.freehand_clear_element.show();
    };

    StudentTools.prototype.repositionFreehandOption = function(x, y, width, height) {
        var left = x + width / 2 + 10;
        var top = y - height / 2 - 10;
        this.freehand_clear_element.css("left", left);
        this.freehand_clear_element.css("top", top);
    };

    StudentTools.prototype.clearNote = function() {
        jQuery.fn.stickyNotes.deleteAll();
    };

    /* disable all student tools and returns focus to html */
    StudentTools.prototype.disableAll = function() {
        // Stop editing of all sticky notes
        jQuery.fn.stickyNotes.stopEditingAll();

        //Change Freehand Mode
        this.freehandMode(false);
        this.focus();
    };

    StudentTools.prototype.repositionFreehand = function() {
        /* reposition freehand dimensions based on html change */
        this.canvas_container.height(this.html_container.height());
        this.canvas.setHeight(this.html_container.height());
    };

    StudentTools.prototype.activeColor = function(color, sticky_class, opcolor) {
        /* change currently active color */

        this.active_color = color;
        this.active_sticky_class = sticky_class;

        this.canvas.freeDrawingBrush.color = color;
    };

    StudentTools.prototype.addNote = function() {
        jQuery.fn.stickyNotes.createNote(this.active_sticky_class);
    };

    StudentTools.prototype.exportFreehand = function() {
        var export_data = JSON.stringify(this.canvas.toObject());
        return export_data;
    };

    StudentTools.prototype.importFreehand = function(data) {
        this.canvas.loadFromJSON(data);
        this.canvas.renderAll();
    };

    StudentTools.prototype.exportNote = function() {
        return jQuery.fn.stickyNotes.exportJSON();
    };

    StudentTools.prototype.importNote = function(data) {
        jQuery.fn.stickyNotes.importJSON(JSON.stringify(data));
    };

    /*
     Saves the changes to the state in the browser
     */
    StudentTools.prototype.saveState = function(type, initial, no_save) {
        var state_object = {
            type: type,
            freehand: undefined,
            note: undefined,
            initial: undefined
        };
        switch (type) {
            case "freehand":
                if (this.temp_canvas_state != "") {
                    state_object.freehand = this.temp_canvas_state;
                    this.temp_canvas_state = "";
                }
                else if (this.temp_before_move != "") {
                    state_object.freehand = this.temp_before_move;
                    this.temp_before_move = "";
                }
                else
                    state_object.freehand = this.exportFreehand();
                break;
            case "notes":
                state_object.note = this.exportNote();
                break;
            default:
                state_object.freehand = this.exportFreehand();
                state_object.note = this.exportNote();
        }

        // check is this for questioning state
        if (this.exists(no_save)) {
            return state_object;
        }

        if (this.exists(initial)) {
            state_object.initial = true;
        }
        this.state_stack.push(state_object);
    };

    /*
     Saves the current state to the server.
     */
    StudentTools.prototype.save = function(url) {
        var freehandEmpty = '{"objects":[],"background":""}';
        var noteEmpty = "[]";
        var delState = undefined;

        var freehandState = this.exportFreehand();
        if (freehandState == freehandEmpty) freehandState = freehandEmpty;

        var noteState = this.exportNote();
        if (noteState == noteEmpty) noteState = noteEmpty;

        var _this = this;
        $.ajax({
            type: "POST",
            contentType: "application/json",
            url: "/api/v1/courseware/concept/state/",
            data: JSON.stringify({
                scope: "user/tools",
                course_instance_id: window.globals.courseInstanceId,
                concept_id: window.globals.concept,
                state: {
                    freehand: freehandState,
                    note: noteState
                },
                delete: delState
            }),
            processData: false,
            success: function() {
                // check if redirect required
                if (_this.exists(url))
                    window.location = url;
                var data = JSON.parse(this.data);
               _this.state_stack.updateInitial(data.state);
                return true;
            },
            error: function(xhr, ajaxOptions, thrownError) {
                return false;
            }
        });
    };

    StudentTools.prototype.freehandCallback = function() {

        var _this = this;
        this.temp_canvas_state = "";
        this.temp_before_move = "";

        // canvas callbacks
        this.canvas.on('object:added', function(options) {
            if (_this.canvas.isDrawingMode) {
                if (options.target.height != 0)
                    _this.saveState("freehand");
                else
                    _this.canvas.remove(options.target);
            }
        });

        this.canvas.on('object:modified', function(options) {
            if (_this.canvas.isDrawingMode) {
                _this.saveState("freehand");
            }
        });

        this.canvas.on('mouse:down', function(options) {
            if (!_this.canvas.getActiveObject() && !_this.canvas.getActiveGroup())
                _this.temp_canvas_state = _this.exportFreehand();
        });

        this.canvas.on('object:selected', function(options) {
            _this.canvas.getActiveObject().hasControls = false;
            _this.showFreehandOption("object");
        });

        this.canvas.on('selection:created', function(options) {
            _this.showFreehandOption("group");
            _this.temp_before_move = _this.exportFreehand();
            _this.canvas.getActiveGroup().hasControls = false;
            _this.canvas.renderAll(false);
        });
        this.canvas.on('selection:cleared', function(options) {
            _this.freehand_clear_element.hide();
            _this.saveState("freehand")
        });
        this.canvas.on('object:moving', function(options) {
            _this.repositionFreehandOption(options.target.left, options.target.top, options.target.width, options.target.height);
        });
    };

    StudentTools.prototype.notesCallback = function(obj, event) {
        if (event == 'editNote') {
            this.toolsObject.disableAll();
        }
        obj.saveState("notes");
    };

    StudentTools.prototype.changeColor = function(element) {
        this.activeColor(element.data("color"), element.data("sticky"), element.data("opcolor"));
    };

    return StudentTools;
});